/*****************************************************************************
**
**  weld.c
**
**  Description:
**     Contains Unigraphics entry points for the application.
**
*****************************************************************************/

/*  Include files */
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <uf.h>
#include <uf_ui.h>
#include <uf_defs.h>
#include <uf_drf.h>
#include <uf_curve.h>
#include <uf_obj.h>
#include <uf_object_types.h>
#include <uf_disp.h>
#include <uf_modl.h>
#include <uf_view.h>
#include <uf_csys.h>
#include <uf_draw.h>
#include <uf_part.h>
#include <uf_vec.h>
#include <uf_trns.h>
#include <uf_mtx.h>
#include <uf_assem.h>
#include "weld.h"

/*****************************************************************************
**  Activation Methods
*****************************************************************************/

/*  Explicit Activation
**      This entry point is used to activate the application explicitly, as in
**      "File->Execute UG/Open->User Function..." */
extern DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
	
	int lines_of_text = 7;
    char label_text[10][MAX_LINE_LENGTH + 1];
    double cp[3], pos[3];
    int i = 0;
    tag_t label;
    UF_DRF_object_t curve;

	int  mpi_array[100];
    double mpr_array[70];
    char rad_symbol[27];
    char dia_symbol[27];
    tag_t part = NULL_TAG;
	
    char symbol_file_name[SYMBOL_FILE_NAME_SIZE + 1] = "weld";
    char symbol_name[SYMBOL_NAME_SIZE + 1] = "TEST";
    UF_DRF_symbol_create_data_t symbol_data;
    double  symbol_angle = 0;
    double  symbol_length = 3.0;
    double  symbol_height = 1.0;
    tag_t   point1_tag, symbol_tag;
    double  label_origin[3];
	 double	point_coord[3];
	 double label_angle;
	 char tmp_str[265];
    UF_DRF_label_info_t  *label_info;
 	
    /* Initialize the API environment */
    int errorCode = UF_initialize();
	
    if ( 0 == errorCode )
    {
		part = UF_ASSEM_ask_work_part();
		if(part == NULL_TAG) 
		{
			uc1601("No work part", TRUE);
			return;
		}
		/* create a label */
		/*strcpy(text[0], "This is a LABEL.");
		strcpy(text[1], "It points to the screen position at (20,5).");
		strcpy(text[2], "The next line is BLANK.");
		strcpy(text[3], "");
		strcpy(text[4], "The next line is very long....");
		strcpy(text[5], "ABCDEFGHIJKLMNOPQRSTUVWXYZYXWV");
		strcpy(text[6], "This is the LAST line.");
		object.object_tag = base_pt_tag;
		object.object_view_tag = NULL_TAG;
		object.object_assoc_type = UF_DRF_end_point;
		object.object_assoc_modifier = UF_DRF_first_end_point;
		errorCode = UF_DRF_create_label(lines_of_text, text, origin,
		UF_DRF_leader_attach_screen, &object, base_pt,
		&text_aid_tag);*/
		
        UF_DRF_set_specified_sbf_file(symbol_file_name);
        UF_DRF_ask_sbf_file(symbol_file_name);
//		  uc1601(symbol_file_name,TRUE);
        if(!strcmp(symbol_file_name,"ug_default.sbf")) 
        {
           uc1601("   weld.sbf",1);
           return;
        }

		UF_DRF_init_object_structure(&curve);
		
		while(select_drf_curve_pos("Select item to label", &curve, cp))
		{
            if(select_pos("Indicate origin for label", pos))
            {
                map_abs2wcs(pos);  /* required if view is model view */
                sprintf(&label_text[0][0], "Label %d", ++i);
                errorCode = UF_DRF_create_label(1, label_text, pos,	UF_DRF_leader_attach_object, &curve, cp, &label);
                if (errorCode == 0)
                {
                    errorCode = UF_DRF_ask_object_preferences(label, mpi_array, mpr_array, rad_symbol, dia_symbol);
                    if (errorCode == 0)
                    {
                        mpi_array[29]=6;  /* Set Arrowhead1 Type  6 = None  */
                        errorCode = UF_DRF_set_object_preferences(label,mpi_array, mpr_array, rad_symbol, dia_symbol );
                        if (errorCode == 0)
                        {
                          UF_DRF_ask_label_info(label,
                                            &label_origin [0],
                                            &label_info);
								  label_angle = label_info->arrow_info->arrow_angle;
								  sprintf(tmp_str, "ANGLE = %.3lf", label_angle*180.0/PI);
								  uc1601(tmp_str, TRUE);
									
									point_coord[0] = label_info->arrow_info->arrow_origin[0]+symbol_length*cos(label_angle);
									point_coord[1] = label_info->arrow_info->arrow_origin[1]+symbol_length*sin(label_angle);
									point_coord[2] = label_info->arrow_info->arrow_origin[2];
                          UF_CURVE_create_point (point_coord, &point1_tag);
                          strcpy (symbol_data.symbol_name, symbol_name);
                          symbol_data.angle      = label_angle*180.0/PI+180.0;
                          symbol_data.length     = symbol_length;
                          symbol_data.height     = symbol_height;
                          symbol_data.anchor_tag = point1_tag;
//                        errorCode = UF_DRF_add_symbol_to_object(&symbol_data, label);
								errorCode = UF_DRF_place_symbol(&symbol_data, TRUE, TRUE, &symbol_tag);

                          UF_OBJ_delete_object(point1_tag);
                          UF_DRF_free_label( &label_info );
                        }
                    }
                }
                
            }
            
            //fix_it(cp, curve.object_view_tag);  /* may help on dwg */
            
            /*sprintf(&label_text[0][0], "Label %d", ++i);
            UF_DRF_create_label(1, label_text, pos,
            UF_DRF_leader_attach_object, &curve, cp, &label);*/
		}
		
		/* Print out any error messages */
		PrintErrorMessage( errorCode );
		
		/* Terminate the API environment */
		errorCode = UF_terminate();
	}
    /* Print out any error messages */
    PrintErrorMessage( errorCode );
}

static void map_abs2view(double c[3])
{
    char
        vname[30];
    double
        abs[9] = { 0,0,0, 1,0,0, 0,1,0 },
        vw[12] = { 0,0,0 },
        mx[12];
    int
        irc;

    uc1653(vname);
    uc6433(vname, &vw[3]);

    FTN(uf5940)(abs, vw, mx, &irc);
    FTN(uf5941)(c, mx);
}

static void map_view2abs(double c[3])
{
    char
        vname[30];
    double
        abs[9] = { 0,0,0, 1,0,0, 0,1,0 },
        vw[12] = { 0,0,0 },
        mx[12];
    int
        irc;

    uc1653(vname);
    uc6433(vname, &vw[3]);

    FTN(uf5940)(vw, abs, mx, &irc);
    FTN(uf5941)(c, mx);

}

static void ask_pos_on_obj(tag_t obj, double loc[3])
{
    tag_t
        line;
    double
        cp[3],
        dist;
    UF_CURVE_line_t
        lp;

    map_abs2view(loc);

    lp.start_point[0] = loc[0];
    lp.start_point[1] = loc[1];
    lp.start_point[2] = loc[2] + 1000;
    lp.end_point[0] = loc[0];
    lp.end_point[1] = loc[1];
    lp.end_point[2] = loc[2] - 1000;

    map_view2abs(lp.start_point);
    map_view2abs(lp.end_point);

    UF_CURVE_create_line(&lp, &line);
    UF_MODL_ask_minimum_dist(obj, line, 0, cp, 0, cp, &dist, loc, cp);

    UF_OBJ_delete_object(line);
}

static int mask_for_drf_curves(UF_UI_selection_p_t select, void *type)
{
    UF_UI_mask_t
        mask[7] = { { UF_line_type, 0, 0 },
                    { UF_circle_type, 0, 0 },
                    { UF_conic_type, 0, 0 },
                    { UF_spline_type, 0, 0 },
                    { UF_solid_type, 0, UF_UI_SEL_FEATURE_ANY_EDGE },
                    { UF_solid_silhouette_type, 0, 0 },
                    { UF_section_edge_type, 0, 0 }};

    if (!UF_UI_set_sel_mask(select,
            UF_UI_SEL_MASK_CLEAR_AND_ENABLE_SPECIFIC, 7, mask))
        return (UF_UI_SEL_SUCCESS);
    else
        return (UF_UI_SEL_FAILURE);
}

static logical select_drf_curve_pos(char *prompt, UF_DRF_object_p_t curve,
    double cp[3])
{
    int
        resp;

    UF_UI_set_cursor_view(0);
    UF_UI_select_with_single_dialog(prompt, "",
        UF_UI_SEL_SCOPE_WORK_PART, mask_for_drf_curves, NULL, &resp,
        &curve->object_tag, cp, &curve->object_view_tag);
    UF_UI_set_cursor_view(1);

    if (resp == UF_UI_OBJECT_SELECTED || resp == UF_UI_OBJECT_SELECTED_BY_NAME)
    {
        ask_pos_on_obj(curve->object_tag, cp);
        UF_DISP_set_highlight(curve->object_tag, 0);
        return TRUE;
    }
    else return FALSE;

}

static logical select_pos(char *prompt, double pos[3])
{
    int
        resp = uc1615(prompt, pos);

    if (resp == 5) return TRUE;
    else return FALSE;

}

static void map_abs2wcs(double cc[3])
{
    double
        abs[9] = { 0,0,0, 1,0,0, 0,1,0 },
        wcs[12],
        mx[12];
    int
        irc;
    tag_t
        wcs_ent,
        wcs_mx;

    UF_CSYS_ask_wcs(&wcs_ent);
    UF_CSYS_ask_csys_info(wcs_ent, &wcs_mx, wcs);
    UF_CSYS_ask_matrix_values(wcs_mx, &wcs[3]);

    FTN(uf5940)(abs, wcs, mx, &irc);
    FTN(uf5941)(cc, mx);

}

static void fix_it(double pos[3], tag_t member_view)
{
    double
        dwg_ref[3],
        dsize[2],
        factor = 1,
        mdl_ref[3],
        mx[9],
        scale,
        tmp_pos[3];
    tag_t
        dsp_prt_tag,
        pnt;
    int
        dcode,
        dunits,
        n,
        punits;
    char
        cur_dwg[MAX_ENTITY_NAME_SIZE] = { "" },
        member_view_name[MAX_ENTITY_NAME_SIZE];

    /* Get the view reference point and scale */
    uc5027(member_view, member_view_name, &n);
    uc6442(member_view_name, mdl_ref, &scale);

    /* Get the drawing reference point */
    uc6483(cur_dwg, member_view_name, dwg_ref);

    /* Get view rotation matrix */
    uc6433(member_view_name, mx);

    /* Get drawing units (1 = inches, 2 = mm) */
    uc6479(cur_dwg, &dunits, &dcode, dsize);
    if (dunits == 2) factor = 25.4;

    /* Get part units (1 = mm, 2 = inches) */
    dsp_prt_tag = UF_PART_ask_display_part( );
    UF_PART_ask_units( dsp_prt_tag, &punits );
    if (punits == 1) factor = factor / 25.4;

    /* Create a point on the drawing */
    UF_CURVE_create_point(pos, &pnt);

    /* Convert to drawing units */
    UF_VEC3_scale(factor, pos, pos);

    /* Add the drawing reference point */
    UF_VEC3_add(pos, dwg_ref, pos);

    /* Convert back to part units */
    UF_VEC3_scale(1/factor, pos, pos);

    /* Map through the transform of the view rotation matrix */
    UF_MTX3_vec_multiply_t(pos, mx, tmp_pos);

    /* Scale down by the view's drawing scale */
    UF_VEC3_scale(1/scale, tmp_pos, pos);

    /* Add the view reference point */
    UF_VEC3_add(pos, mdl_ref, pos);

    /* Create a point in the member view */
    UF_CURVE_create_point(pos, &pnt);
    uc6401("", pnt);
    uc6402(member_view_name, pnt);


}






/*****************************************************************************
**  Utilities
*****************************************************************************/

/* Unload Handler
**     This function specifies when to unload your application from Unigraphics.
**     If your application registers a callback (from a MenuScript item or a
**     User Defined Object for example), this function MUST return
**     "UF_UNLOAD_UG_TERMINATE". */
extern int ufusr_ask_unload( void )
{
    return( UF_UNLOAD_IMMEDIATELY );
}

/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        uc1601( message,1 );
    }
}
